@@ -1,7 +1,7 @@
 /*
  * wee-arraylist.c - array lists management
  *
- * Copyright (C) 2014-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,6 +32,31 @@
 #include "wee-string.h"
 
 
+/*
+ * Compares two arraylist entries (default comparator).
+ * It just compares pointers.
+ *
+ * Returns:
+ *   -1: pointer1 < pointer2
+ *    0: pointer1 == pointer2
+ *    1: pointer1 > pointer2
+ */
+
+int
+arraylist_cmp_default_cb (void *data, struct t_arraylist *arraylist,
+                          void *pointer1, void *pointer2)
+{
+    /* make C compiler happy */
+    (void) data;
+    (void) arraylist;
+
+    if (pointer1 < pointer2)
+        return -1;
+    if (pointer1 > pointer2)
+        return 1;
+    return 0;
+}
+
 /*
  * Creates a new arraylist.
  *
@@ -48,7 +73,7 @@ arraylist_new (int initial_size,
     struct t_arraylist *new_arraylist;
 
     /* check arguments */
-    if ((initial_size < 0) || !callback_cmp)
+    if (initial_size < 0)
         return NULL;
 
     new_arraylist = malloc (sizeof (*new_arraylist));
@@ -76,8 +101,10 @@ arraylist_new (int initial_size,
     }
     new_arraylist->sorted = sorted;
     new_arraylist->allow_duplicates = allow_duplicates;
-    new_arraylist->callback_cmp = callback_cmp;
-    new_arraylist->callback_cmp_data = callback_cmp_data;
+    new_arraylist->callback_cmp = (callback_cmp) ?
+        callback_cmp : &arraylist_cmp_default_cb;
+    new_arraylist->callback_cmp_data = (callback_cmp) ?
+        callback_cmp_data : NULL;
     new_arraylist->callback_free = callback_free;
     new_arraylist->callback_free_data = callback_free_data;
 
@@ -610,26 +637,28 @@ arraylist_clear (struct t_arraylist *arraylist)
         }
     }
 
-    if (arraylist->data
-        && (arraylist->size_alloc != arraylist->size_alloc_min))
+    if (arraylist->data)
     {
-        free (arraylist->data);
-        arraylist->data = NULL;
-        arraylist->size_alloc = 0;
-        if (arraylist->size_alloc_min > 0)
+        if (arraylist->size_alloc != arraylist->size_alloc_min)
         {
-            arraylist->data = calloc(arraylist->size_alloc_min,
-                                     sizeof (*arraylist->data));
-            if (!arraylist->data)
-                return 0;
-            arraylist->size_alloc = arraylist->size_alloc_min;
+            free (arraylist->data);
+            arraylist->data = NULL;
+            arraylist->size_alloc = 0;
+            if (arraylist->size_alloc_min > 0)
+            {
+                arraylist->data = calloc (arraylist->size_alloc_min,
+                                          sizeof (*arraylist->data));
+                if (!arraylist->data)
+                    return 0;
+                arraylist->size_alloc = arraylist->size_alloc_min;
+            }
+        }
+        else if (arraylist->size_alloc > 0)
+        {
+            memset (arraylist->data,
+                    0,
+                    arraylist->size_alloc * sizeof (*arraylist->data));
         }
-    }
-    else if (arraylist->size_alloc > 0)
-    {
-        memset (arraylist->data,
-                0,
-                arraylist->size_alloc * sizeof (*arraylist->data));
     }
 
     arraylist->size = 0;
@@ -674,7 +703,6 @@ arraylist_print_log (struct t_arraylist *arraylist, const char *name)
 {
     int i;
 
-    log_printf ("");
     log_printf ("[arraylist %s (addr:0x%lx)]", name, arraylist);
     log_printf ("  size . . . . . . . . . : %d", arraylist->size);
     log_printf ("  size_alloc . . . . . . : %d", arraylist->size_alloc);
