@@ -1,7 +1,7 @@
 /*
  * irc-bar-item.c - bar items for IRC plugin
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,13 +16,15 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
+#include <gnutls/gnutls.h>
+
 #include "../weechat-plugin.h"
 #include "irc.h"
 #include "irc-buffer.h"
@@ -33,66 +35,13 @@
 #include "irc-nick.h"
 
 
-/*
- * Returns content of bar item "away": bar item with away indicator.
- */
-
-char *
-irc_bar_item_away (void *data, struct t_gui_bar_item *item,
-                   struct t_gui_window *window, struct t_gui_buffer *buffer,
-                   struct t_hashtable *extra_info)
-{
-    struct t_irc_server *server;
-    char *buf, *message;
-    int length;
-
-    /* make C compiler happy */
-    (void) data;
-    (void) item;
-    (void) window;
-    (void) extra_info;
-
-    if (!buffer)
-        return NULL;
-
-    buf = NULL;
-
-    irc_buffer_get_server_and_channel (buffer, &server, NULL);
-
-    if (server && server->is_away)
-    {
-        if (weechat_config_boolean (irc_config_look_item_away_message)
-            && server->away_message && server->away_message[0])
-        {
-            message = strdup (server->away_message);
-        }
-        else
-        {
-            message = strdup (_("away"));
-        }
-        if (message)
-        {
-            length = strlen (message) + 64 + 1;
-            buf = malloc (length);
-            if (buf)
-            {
-                snprintf (buf, length, "%s%s",
-                          IRC_COLOR_ITEM_AWAY,
-                          message);
-            }
-            free (message);
-        }
-    }
-
-    return buf;
-}
-
 /*
  * Returns content of bar item "buffer_plugin": bar item with buffer plugin.
  */
 
 char *
-irc_bar_item_buffer_plugin (void *data, struct t_gui_bar_item *item,
+irc_bar_item_buffer_plugin (const void *pointer, void *data,
+                            struct t_gui_bar_item *item,
                             struct t_gui_window *window,
                             struct t_gui_buffer *buffer,
                             struct t_hashtable *extra_info)
@@ -104,6 +53,7 @@ irc_bar_item_buffer_plugin (void *data, struct t_gui_bar_item *item,
     struct t_irc_channel *channel;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) item;
     (void) window;
@@ -166,7 +116,7 @@ irc_bar_item_buffer_plugin (void *data, struct t_gui_bar_item *item,
 char *
 irc_bar_item_buffer_name_content (struct t_gui_buffer *buffer, int short_name)
 {
-    char buf[512], buf_name[256], modes[128];
+    char buf[512], buf_name[256];
     const char *name, *localvar_type;
     int part_from_channel, display_server, is_channel;
     struct t_irc_server *server;
@@ -176,7 +126,6 @@ irc_bar_item_buffer_name_content (struct t_gui_buffer *buffer, int short_name)
         return NULL;
 
     buf_name[0] = '\0';
-    modes[0] = '\0';
 
     display_server = (weechat_config_integer (irc_config_look_item_display_server) == IRC_CONFIG_LOOK_ITEM_DISPLAY_SERVER_NAME);
 
@@ -239,10 +188,10 @@ irc_bar_item_buffer_name_content (struct t_gui_buffer *buffer, int short_name)
         }
     }
 
-    snprintf (buf, sizeof (buf), "%s%s%s",
+    snprintf (buf, sizeof (buf),
+              "%s%s",
               (server && server->ssl_connected) ? IRC_COLOR_STATUS_NAME_SSL : IRC_COLOR_STATUS_NAME,
-              buf_name,
-              modes);
+              buf_name);
 
     return strdup (buf);
 }
@@ -252,12 +201,14 @@ irc_bar_item_buffer_name_content (struct t_gui_buffer *buffer, int short_name)
  */
 
 char *
-irc_bar_item_buffer_name (void *data, struct t_gui_bar_item *item,
+irc_bar_item_buffer_name (const void *pointer, void *data,
+                          struct t_gui_bar_item *item,
                           struct t_gui_window *window,
                           struct t_gui_buffer *buffer,
                           struct t_hashtable *extra_info)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) item;
     (void) window;
@@ -272,12 +223,14 @@ irc_bar_item_buffer_name (void *data, struct t_gui_bar_item *item,
  */
 
 char *
-irc_bar_item_buffer_short_name (void *data, struct t_gui_bar_item *item,
+irc_bar_item_buffer_short_name (const void *pointer, void *data,
+                                struct t_gui_bar_item *item,
                                 struct t_gui_window *window,
                                 struct t_gui_buffer *buffer,
                                 struct t_hashtable *extra_info)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) item;
     (void) window;
@@ -291,7 +244,8 @@ irc_bar_item_buffer_short_name (void *data, struct t_gui_bar_item *item,
  */
 
 char *
-irc_bar_item_buffer_modes (void *data, struct t_gui_bar_item *item,
+irc_bar_item_buffer_modes (const void *pointer, void *data,
+                           struct t_gui_bar_item *item,
                            struct t_gui_window *window,
                            struct t_gui_buffer *buffer,
                            struct t_hashtable *extra_info)
@@ -303,6 +257,7 @@ irc_bar_item_buffer_modes (void *data, struct t_gui_bar_item *item,
     struct t_irc_channel *channel;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) item;
     (void) window;
@@ -326,7 +281,7 @@ irc_bar_item_buffer_modes (void *data, struct t_gui_bar_item *item,
         modes_without_args = NULL;
         if (!irc_config_display_channel_modes_arguments (channel->modes))
         {
-            pos_space = strchr(channel->modes, ' ');
+            pos_space = strchr (channel->modes, ' ');
             if (pos_space)
             {
                 modes_without_args = weechat_strndup (
@@ -351,7 +306,8 @@ irc_bar_item_buffer_modes (void *data, struct t_gui_bar_item *item,
  */
 
 char *
-irc_bar_item_channel (void *data, struct t_gui_bar_item *item,
+irc_bar_item_channel (const void *pointer, void *data,
+                      struct t_gui_bar_item *item,
                       struct t_gui_window *window, struct t_gui_buffer *buffer,
                       struct t_hashtable *extra_info)
 {
@@ -362,6 +318,7 @@ irc_bar_item_channel (void *data, struct t_gui_bar_item *item,
     struct t_irc_channel *channel;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) item;
     (void) window;
@@ -423,12 +380,113 @@ irc_bar_item_channel (void *data, struct t_gui_bar_item *item,
     return strdup (buf);
 }
 
+/*
+ * Returns content of bar item "irc_nick": bar item with nick name
+ * (without modes).
+ */
+
+char *
+irc_bar_item_nick (const void *pointer, void *data,
+                   struct t_gui_bar_item *item,
+                   struct t_gui_window *window, struct t_gui_buffer *buffer,
+                   struct t_hashtable *extra_info)
+{
+    char buf[512];
+    struct t_irc_server *server;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) item;
+    (void) window;
+    (void) extra_info;
+
+    if (!buffer)
+        return NULL;
+
+    irc_buffer_get_server_and_channel (buffer, &server, NULL);
+    if (!server || !server->nick)
+        return NULL;
+
+    snprintf (buf, sizeof (buf), "%s%s",
+              IRC_COLOR_INPUT_NICK,
+              server->nick);
+
+    return strdup (buf);
+}
+
+/*
+ * Returns content of bar item "irc_host": bar item with self host.
+ */
+
+char *
+irc_bar_item_host (const void *pointer, void *data,
+                   struct t_gui_bar_item *item,
+                   struct t_gui_window *window, struct t_gui_buffer *buffer,
+                   struct t_hashtable *extra_info)
+{
+    struct t_irc_server *server;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) item;
+    (void) window;
+    (void) extra_info;
+
+    if (!buffer)
+        return NULL;
+
+    irc_buffer_get_server_and_channel (buffer, &server, NULL);
+    if (!server || !server->host)
+        return NULL;
+
+    return strdup (server->host);
+}
+
+/*
+ * Returns content of bar item "irc_nick_host": bar item with nick and host.
+ */
+
+char *
+irc_bar_item_nick_host (const void *pointer, void *data,
+                        struct t_gui_bar_item *item,
+                        struct t_gui_window *window,
+                        struct t_gui_buffer *buffer,
+                        struct t_hashtable *extra_info)
+{
+    char buf[512];
+    struct t_irc_server *server;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) item;
+    (void) window;
+    (void) extra_info;
+
+    if (!buffer)
+        return NULL;
+
+    irc_buffer_get_server_and_channel (buffer, &server, NULL);
+    if (!server || !server->nick)
+        return NULL;
+
+    snprintf (buf, sizeof (buf), "%s%s%s",
+              server->nick,
+              (server->host) ? "!" : "",
+              (server->host) ? server->host : "");
+
+    return strdup (buf);
+}
+
 /*
  * Returns content of bar item "lag": bar item with lag value.
  */
 
 char *
-irc_bar_item_lag (void *data, struct t_gui_bar_item *item,
+irc_bar_item_lag (const void *pointer, void *data,
+                  struct t_gui_bar_item *item,
                   struct t_gui_window *window, struct t_gui_buffer *buffer,
                   struct t_hashtable *extra_info)
 {
@@ -436,6 +494,7 @@ irc_bar_item_lag (void *data, struct t_gui_bar_item *item,
     struct t_irc_server *server;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) item;
     (void) window;
@@ -462,12 +521,75 @@ irc_bar_item_lag (void *data, struct t_gui_bar_item *item,
     return NULL;
 }
 
+/*
+ * Returns content of bar item "tls_version": bar item with TLS version value.
+ */
+
+char *
+irc_bar_item_tls_version (const void *pointer, void *data,
+                          struct t_gui_bar_item *item,
+                          struct t_gui_window *window, struct t_gui_buffer *buffer,
+                          struct t_hashtable *extra_info)
+{
+    char buf[128];
+    struct t_irc_server *server;
+    gnutls_protocol_t version;
+    const char *color;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) item;
+    (void) window;
+    (void) extra_info;
+
+    if (!buffer)
+        return NULL;
+
+    irc_buffer_get_server_and_channel (buffer, &server, NULL);
+    if (server && server->is_connected)
+    {
+        if (server->ssl_connected)
+        {
+            version = gnutls_protocol_get_version (server->gnutls_sess);
+            switch (version)
+            {
+#if LIBGNUTLS_VERSION_NUMBER >= 0x030603 /* 3.6.3 */
+                case GNUTLS_TLS1_3:
+                    color = IRC_COLOR_ITEM_TLS_VERSION_OK;
+                    break;
+#endif
+                case GNUTLS_TLS1_2:
+                    color = IRC_COLOR_ITEM_TLS_VERSION_DEPRECATED;
+                    break;
+                default:
+                    color = IRC_COLOR_ITEM_TLS_VERSION_INSECURE;
+            }
+            snprintf (buf, sizeof (buf), "%s%s", color,
+                      gnutls_protocol_get_name (version));
+            return strdup (buf);
+        }
+        else
+        {
+            snprintf (buf, sizeof (buf), "%s%s",
+                      IRC_COLOR_ITEM_TLS_VERSION_INSECURE,
+                      /* TRANSLATORS: this is displayed in a bar item and must be as short as possible */
+                      _("cleartext"));
+            return strdup (buf);
+        }
+    }
+
+    return NULL;
+}
+
+
 /*
  * Returns content of bar item "input_prompt": bar item with input prompt.
  */
 
 char *
-irc_bar_item_input_prompt (void *data, struct t_gui_bar_item *item,
+irc_bar_item_input_prompt (const void *pointer, void *data,
+                           struct t_gui_bar_item *item,
                            struct t_gui_window *window,
                            struct t_gui_buffer *buffer,
                            struct t_hashtable *extra_info)
@@ -479,6 +601,7 @@ irc_bar_item_input_prompt (void *data, struct t_gui_bar_item *item,
     int length;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) item;
     (void) window;
@@ -548,7 +671,8 @@ irc_bar_item_input_prompt (void *data, struct t_gui_bar_item *item,
  */
 
 char *
-irc_bar_item_nick_modes (void *data, struct t_gui_bar_item *item,
+irc_bar_item_nick_modes (const void *pointer, void *data,
+                         struct t_gui_bar_item *item,
                          struct t_gui_window *window,
                          struct t_gui_buffer *buffer,
                          struct t_hashtable *extra_info)
@@ -558,6 +682,7 @@ irc_bar_item_nick_modes (void *data, struct t_gui_bar_item *item,
     int length;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) item;
     (void) window;
@@ -582,19 +707,71 @@ irc_bar_item_nick_modes (void *data, struct t_gui_bar_item *item,
     return buf;
 }
 
+/*
+ * Returns content of bar item "nick_prefix": bar item with nick prefix.
+ */
+
+char *
+irc_bar_item_nick_prefix (const void *pointer, void *data,
+                          struct t_gui_bar_item *item,
+                          struct t_gui_window *window,
+                          struct t_gui_buffer *buffer,
+                          struct t_hashtable *extra_info)
+{
+    struct t_irc_server *server;
+    struct t_irc_channel *channel;
+    struct t_irc_nick *ptr_nick;
+    char str_prefix[64];
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) item;
+    (void) window;
+    (void) extra_info;
+
+    if (!buffer)
+        return NULL;
+
+    irc_buffer_get_server_and_channel (buffer, &server, &channel);
+    if (!server || !server->nick)
+        return NULL;
+
+    str_prefix[0] = '\0';
+    if (channel && (channel->type == IRC_CHANNEL_TYPE_CHANNEL))
+    {
+        ptr_nick = irc_nick_search (server, channel, server->nick);
+        if (ptr_nick)
+        {
+            if (weechat_config_boolean (irc_config_look_nick_mode_empty)
+                || (ptr_nick->prefix[0] != ' '))
+            {
+                snprintf (str_prefix, sizeof (str_prefix), "%s%s",
+                          weechat_color (
+                              irc_nick_get_prefix_color_name (
+                                  server, ptr_nick->prefix[0])),
+                          ptr_nick->prefix);
+            }
+        }
+    }
+
+    return (str_prefix[0]) ? strdup (str_prefix) : NULL;
+}
+
 /*
  * Focus on nicklist.
  */
 
 struct t_hashtable *
-irc_bar_item_focus_buffer_nicklist (void *data,
+irc_bar_item_focus_buffer_nicklist (const void *pointer, void *data,
                                     struct t_hashtable *info)
 {
-    long unsigned int value;
+    unsigned long value;
     int rc;
     struct t_gui_buffer *buffer;
     struct t_irc_nick *ptr_nick;
     const char *str_buffer, *nick;
+    char str_value[128];
 
     str_buffer = weechat_hashtable_get (info, "_buffer");
     if (!str_buffer || !str_buffer[0])
@@ -609,6 +786,7 @@ irc_bar_item_focus_buffer_nicklist (void *data,
     IRC_BUFFER_GET_SERVER_CHANNEL(buffer);
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     if (ptr_server && ptr_channel)
@@ -617,9 +795,15 @@ irc_bar_item_focus_buffer_nicklist (void *data,
         if (nick)
         {
             ptr_nick = irc_nick_search (ptr_server, ptr_channel, nick);
-            if (ptr_nick && ptr_nick->host)
+            if (ptr_nick)
             {
-                weechat_hashtable_set (info, "irc_host", ptr_nick->host);
+                snprintf (str_value, sizeof (str_value),
+                          "0x%lx", (unsigned long)ptr_nick);
+                weechat_hashtable_set (info, "irc_nick", str_value);
+
+                if (ptr_nick->host)
+                    weechat_hashtable_set (info, "irc_host", ptr_nick->host);
+
                 return info;
             }
         }
@@ -633,10 +817,12 @@ irc_bar_item_focus_buffer_nicklist (void *data,
  */
 
 int
-irc_bar_item_buffer_switch (void *data, const char *signal,
+irc_bar_item_buffer_switch (const void *pointer, void *data,
+                            const char *signal,
                             const char *type_data, void *signal_data)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) signal;
     (void) type_data;
@@ -650,6 +836,10 @@ irc_bar_item_buffer_switch (void *data, const char *signal,
     weechat_bar_item_update ("lag");
     weechat_bar_item_update ("input_prompt");
     weechat_bar_item_update ("irc_nick_modes");
+    weechat_bar_item_update ("irc_nick");
+    weechat_bar_item_update ("irc_host");
+    weechat_bar_item_update ("irc_nick_host");
+    weechat_bar_item_update ("tls_version");
 
     return WEECHAT_RC_OK;
 }
@@ -673,17 +863,36 @@ irc_bar_item_update_channel ()
 void
 irc_bar_item_init ()
 {
-    weechat_bar_item_new ("away", &irc_bar_item_away, NULL);
-    weechat_bar_item_new ("buffer_plugin", &irc_bar_item_buffer_plugin, NULL);
-    weechat_bar_item_new ("buffer_name", &irc_bar_item_buffer_name, NULL);
-    weechat_bar_item_new ("buffer_short_name", &irc_bar_item_buffer_short_name, NULL);
-    weechat_bar_item_new ("buffer_modes", &irc_bar_item_buffer_modes, NULL);
-    weechat_bar_item_new ("irc_channel", &irc_bar_item_channel, NULL);
-    weechat_bar_item_new ("lag", &irc_bar_item_lag, NULL);
-    weechat_bar_item_new ("input_prompt", &irc_bar_item_input_prompt, NULL);
-    weechat_bar_item_new ("irc_nick_modes", &irc_bar_item_nick_modes, NULL);
+    weechat_bar_item_new ("buffer_plugin",
+                          &irc_bar_item_buffer_plugin, NULL, NULL);
+    weechat_bar_item_new ("buffer_name",
+                          &irc_bar_item_buffer_name, NULL, NULL);
+    weechat_bar_item_new ("buffer_short_name",
+                          &irc_bar_item_buffer_short_name, NULL, NULL);
+    weechat_bar_item_new ("buffer_modes",
+                          &irc_bar_item_buffer_modes, NULL, NULL);
+    weechat_bar_item_new ("irc_channel",
+                          &irc_bar_item_channel, NULL, NULL);
+    weechat_bar_item_new ("irc_nick",
+                          &irc_bar_item_nick, NULL, NULL);
+    weechat_bar_item_new ("irc_host",
+                          &irc_bar_item_host, NULL, NULL);
+    weechat_bar_item_new ("irc_nick_host",
+                          &irc_bar_item_nick_host, NULL, NULL);
+    weechat_bar_item_new ("lag",
+                          &irc_bar_item_lag, NULL, NULL);
+    weechat_bar_item_new ("input_prompt",
+                          &irc_bar_item_input_prompt, NULL, NULL);
+    weechat_bar_item_new ("irc_nick_modes",
+                          &irc_bar_item_nick_modes, NULL, NULL);
+    weechat_bar_item_new ("irc_nick_prefix",
+                          &irc_bar_item_nick_prefix, NULL, NULL);
+    weechat_bar_item_new ("tls_version",
+                          &irc_bar_item_tls_version, NULL, NULL);
+
     weechat_hook_focus ("buffer_nicklist",
-                        &irc_bar_item_focus_buffer_nicklist, NULL);
+                        &irc_bar_item_focus_buffer_nicklist, NULL, NULL);
+
     weechat_hook_signal ("buffer_switch",
-                         &irc_bar_item_buffer_switch, NULL);
+                         &irc_bar_item_buffer_switch, NULL, NULL);
 }
